"""
Configuration management for Nage
"""
import json
from pathlib import Path
from typing import Optional, Dict, Any


class Config:
    """Configuration manager for Nage"""
    
    # 预设的 API 端点
    PRESET_ENDPOINTS = {
        "deepseek": "https://api.deepseek.com/chat/completions",
        "openai": "https://api.openai.com/v1/chat/completions",
        "anthropic": "https://api.anthropic.com/v1/messages",
        "moonshot": "https://api.moonshot.cn/v1/chat/completions",
        "zhipu": "https://open.bigmodel.cn/api/paas/v4/chat/completions"
    }
    
    # 支持的语言
    SUPPORTED_LANGUAGES = {
        "zh": "中文",
        "en": "English"
    }
    
    # 默认模型
    DEFAULT_MODELS = {
        "deepseek": "deepseek-chat",
        "openai": "gpt-3.5-turbo",
        "anthropic": "claude-3-haiku-20240307",
        "moonshot": "moonshot-v1-8k",
        "zhipu": "glm-4"
    }
    
    def __init__(self):
        self.config_dir = Path.home() / ".nage"
        self.config_file = self.config_dir / "config.json"
        self.ensure_config_dir()
        
    def ensure_config_dir(self):
        """Ensure the configuration directory exists"""
        self.config_dir.mkdir(exist_ok=True)
        
    def load_config(self) -> Dict[str, Any]:
        """Load configuration from file"""
        if not self.config_file.exists():
            return {}
        
        try:
            with open(self.config_file, 'r') as f:
                return json.load(f)
        except (json.JSONDecodeError, IOError):
            return {}
    
    def save_config(self, config: Dict[str, Any]):
        """Save configuration to file"""
        try:
            with open(self.config_file, 'w') as f:
                json.dump(config, f, indent=2)
        except IOError as e:
            raise RuntimeError(f"Failed to save configuration: {e}")
    
    def get_api_endpoint(self) -> Optional[str]:
        """Get API endpoint from config"""
        config = self.load_config()
        return config.get("api_endpoint")
    
    def set_api_endpoint(self, endpoint: str):
        """Set API endpoint in config"""
        # 检查是否是预设端点的别名
        if endpoint.lower() in self.PRESET_ENDPOINTS:
            endpoint = self.PRESET_ENDPOINTS[endpoint.lower()]
        
        config = self.load_config()
        config["api_endpoint"] = endpoint
        self.save_config(config)
    
    def get_api_key(self) -> Optional[str]:
        """Get API key from config"""
        config = self.load_config()
        return config.get("api_key")
    
    def set_api_key(self, key: str):
        """Set API key in config"""
        config = self.load_config()
        config["api_key"] = key
        self.save_config(config)
    
    def get_language(self) -> str:
        """Get language setting from config"""
        config = self.load_config()
        return config.get("language", "en")  # 默认英文
    
    def set_language(self, language: str):
        """Set language setting in config"""
        if language not in self.SUPPORTED_LANGUAGES:
            raise ValueError(f"Unsupported language: {language}. Supported: {list(self.SUPPORTED_LANGUAGES.keys())}")
        
        config = self.load_config()
        config["language"] = language
        self.save_config(config)
    
    def get_model(self) -> str:
        """Get model setting from config"""
        config = self.load_config()
        return config.get("model", "deepseek-chat")  # 默认模型
    
    def set_model(self, model: str):
        """Set model setting in config"""
        config = self.load_config()
        config["model"] = model
        self.save_config(config)
    
    def get_default_model_for_endpoint(self, endpoint: str) -> str:
        """Get default model for a given endpoint"""
        for name, url in self.PRESET_ENDPOINTS.items():
            if url == endpoint:
                return self.DEFAULT_MODELS.get(name, "deepseek-chat")
        return "deepseek-chat"
    
    def is_configured(self) -> bool:
        """Check if both API endpoint and key are configured"""
        return self.get_api_endpoint() is not None and self.get_api_key() is not None
    
    def get_preset_endpoints(self) -> Dict[str, str]:
        """Get available preset endpoints"""
        return self.PRESET_ENDPOINTS.copy()
    
    def get_supported_languages(self) -> Dict[str, str]:
        """Get supported languages"""
        return self.SUPPORTED_LANGUAGES.copy()
    
    def validate_endpoint(self, endpoint: str) -> bool:
        """Validate if endpoint looks like a valid URL"""
        import re
        url_pattern = r'^https?://[^\s/$.?#].[^\s]*$'
        return bool(re.match(url_pattern, endpoint))



"""
Main entry point for Nage - AI assisted terminal tool
"""

import click
import sys
import subprocess
from typing import List, Dict, Any
from rich.console import Console
from rich.panel import Panel

from .config import Config
from .ai_client import AIClient
from . import __version__

console = Console()
config = Config()

# 多语言支持
MESSAGES = {
    "zh": {
        "needs_more_info": "需要更多信息",
        "description": "描述",
        "answer_questions": "请回答以下问题",
        "ai_reply": "AI 回复",
        "reply": "回复",
        "recommended_commands": "推荐执行的命令",
        "explanation": "说明",
        "warnings": "警告",
        "execute_commands": "是否执行这些命令? (y/n/s for selective)",
        "command_suggestions": "命令建议",
        "commands_cancelled": "命令已取消执行",
        "executing_command": "执行命令",
        "command_success": "命令执行成功",
        "command_failed": "命令执行失败",
        "error_info": "错误信息",
        "execution_error": "执行命令时出错",
        "execute_command": "执行此命令? (y/n/q for quit)",
        "stopped_execution": "已停止执行剩余命令",
        "skipped_command": "跳过此命令",
        "asking_ai": "正在询问AI..."
    },
    "en": {
        "needs_more_info": "More Information Needed",
        "description": "Description",
        "answer_questions": "Please answer the following questions",
        "ai_reply": "AI Reply",
        "reply": "Reply",
        "recommended_commands": "Recommended commands to execute",
        "explanation": "Explanation",
        "warnings": "Warnings",
        "execute_commands": "Execute these commands? (y/n/s for selective)",
        "command_suggestions": "Command Suggestions",
        "commands_cancelled": "Commands cancelled",
        "executing_command": "Executing command",
        "command_success": "Command executed successfully",
        "command_failed": "Command execution failed",
        "error_info": "Error info",
        "execution_error": "Error executing command",
        "execute_command": "Execute this command? (y/n/q for quit)",
        "stopped_execution": "Stopped executing remaining commands",
        "skipped_command": "Skipped this command",
        "asking_ai": "Asking AI..."
    }
}

def get_message(key: str) -> str:
    """Get message in current language"""
    lang = config.get_language()
    return MESSAGES.get(lang, MESSAGES["zh"]).get(key, key)


@click.group(invoke_without_command=True)
@click.option('--set-api', help='Set API endpoint')
@click.option('--set-key', help='Set API key')
@click.option('--set-model', help='Set AI model')
@click.option('--set-lang', help='Set language (zh/en)')
@click.option('--set', 'show_config', is_flag=True, help='Show current configuration')
@click.option('--version', is_flag=True, help='Show version information')
@click.argument('prompt', required=False)
@click.pass_context
def cli(ctx, set_api, set_key, set_model, set_lang, show_config, version, prompt):  # type: ignore
    """Nage - AI assisted terminal tool
    
    Ask AI for help with terminal commands and tools.
    """
    
    # Show version information
    if version:
        console.print(Panel.fit(
            f"[bold blue]Nage[/bold blue] - AI assisted terminal tool\n\n"
            f"[bold]Version:[/bold] {__version__}\n"
            f"[bold]Description:[/bold] AI-powered terminal command suggestions\n\n"
            f"[dim]Project URL:[/dim] https://github.com/0x3st/nage\n"
            f"[dim]License:[/dim] MIT License",
            title="Version Information",
            border_style="blue"
        ))
        return
    
    # Handle configuration options
    if set_api:
        config.set_api_endpoint(set_api) # type: ignore
        console.print(f"API endpoint set to: {set_api}", style="green")
        return
    
    if set_key:
        config.set_api_key(set_key) # type: ignore
        console.print("API key has been set", style="green")
        return
    
    if set_model:
        config.set_model(set_model) # type: ignore
        console.print(f"Model set to: {set_model}", style="green")
        return
    
    if set_lang:
        try:
            config.set_language(set_lang) # type: ignore
            console.print(f"Language set to: {set_lang}", style="green")
        except ValueError as e:
            console.print(f"Error: {str(e)}", style="red")
        return
    
    # Show current configuration
    if show_config:
        api_endpoint = config.get_api_endpoint()
        api_key = config.get_api_key()
        current_model = config.get_model()
        current_language = config.get_language()
        
        # Get language display name
        supported_languages = config.get_supported_languages()
        lang_display = supported_languages.get(current_language, current_language)
        
        # 显示预设端点
        preset_info = "\n[bold]Available Preset Endpoints:[/bold]\n"
        for name, url in config.get_preset_endpoints().items():
            preset_info += f"  {name}: {url}\n"
        
        # 显示支持的语言
        language_info = "\n[bold]Supported Languages:[/bold]\n"
        for code, name in supported_languages.items():
            marker = " (current)" if code == current_language else ""
            language_info += f"  {code}: {name}{marker}\n"
        
        console.print(Panel.fit(
            f"[bold blue]Current Configuration[/bold blue] [dim]v{__version__}[/dim]\n\n"
            f"[bold]API Endpoint:[/bold] {api_endpoint or '[red]Not configured[/red]'}\n"
            f"[bold]API Key:[/bold] {'[green]Configured[/green]' if api_key else '[red]Not configured[/red]'}\n"
            f"[bold]Model:[/bold] {current_model}\n"
            f"[bold]Language:[/bold] {lang_display} ({current_language})\n"
            f"[bold]Config File:[/bold] {config.config_file}\n"
            f"{preset_info}\n"
            f"{language_info}\n"
            f"[dim]Status:[/dim] {'[green]Ready to use[/green]' if config.is_configured() else '[red]Setup required[/red]'}\n\n"
            f"[dim]Usage examples:[/dim]\n"
            f"[dim]  nage --set-api=deepseek[/dim]\n"
            f"[dim]  nage --set-lang=zh[/dim]\n"
            f"[dim]  nage --set-model=gpt-4[/dim]",
            title="Configuration",
            border_style="blue"
        ))
        return
    
    # If no subcommand and no prompt, show help
    if prompt is None:
        console.print(Panel.fit(
            f"[bold blue]Nage[/bold blue] - AI assisted terminal tool [dim]v{__version__}[/dim]\n\n"
            "[bold]Usage:[/bold]\n"
            "  nage --set-api=\"<endpoint>\"   Set API endpoint\n"
            "  nage --set-key=\"<key>\"        Set API key\n"
            "  nage --set-model=\"<model>\"    Set AI model\n"
            "  nage --set-lang=\"<lang>\"      Set language (zh/en)\n"
            "  nage --set                    Show current configuration\n"
            "  nage --version                Show version information\n"
            "  nage \"<prompt>\"               Ask AI for help\n\n"
            "[bold]Examples:[/bold]\n"
            "  nage \"how to find large files\"\n"
            "  nage \"git commit best practices\"\n"
            "  nage \"compress folder with tar\"\n"
            "  nage --set-lang=en\n"
            "  nage --set-model=gpt-4",
            title="Help",
            border_style="blue"
        ))
        return
    
    # Check if configured
    if not config.is_configured():
        console.print(Panel.fit(
            "[bold red]Configuration Required[/bold red]\n\n"
            "Please configure your API endpoint and key first:\n\n"
            "[bold]1.[/bold] Set API endpoint:\n"
            "   nage --set-api=\"https://api.openai.com/v1/chat/completions\"\n\n"
            "[bold]2.[/bold] Set API key:\n"
            "   nage --set-key=\"your-api-key-here\"\n\n"
            "[dim]Your configuration is stored in ~/.nage/config.json[/dim]",
            title="Setup Required",
            border_style="red"
        ))
        sys.exit(1)
    
    # Process the prompt
    try:
        with console.status(f"[bold green]{get_message('asking_ai')}[/bold green]", spinner="dots"):
            ai_client = AIClient(config)
            response = ai_client.send_request(prompt) # type: ignore
        
        # Handle error in response
        if "error" in response:
            console.print(Panel.fit(
                f"[bold red]Error:[/bold red] {response['error']}\n\n"
                "[dim]Please check your API configuration and try again.[/dim]",
                title="Error",
                border_style="red"
            ))
            return
        
        # Handle request for more input
        if response.get("requires_input", False):
            handle_input_request(response)
            return
        
        # Display commands and ask for confirmation
        handle_command_execution(response, prompt) # type: ignore
        
    except Exception as e:
        console.print(Panel.fit(
            f"[bold red]Error:[/bold red] {str(e)}\n\n"
            "[dim]Please check your API configuration and try again.[/dim]",
            title="Error",
            border_style="red"
        ))
        sys.exit(1)


def handle_input_request(response: Dict[str, Any]) -> None:
    """Handle when AI needs more input from user"""
    console.print(Panel.fit(
        f"[bold blue]{get_message('needs_more_info')}[/bold blue]\n\n"
        f"[bold]{get_message('description')}:[/bold] {response.get('description', '')}\n\n"
        f"[bold]{get_message('answer_questions')}:[/bold]\n" +
        '\n'.join(f"  {i+1}. {q}" for i, q in enumerate(response.get('input_prompts', []))),
        title=get_message('needs_more_info'),
        border_style="blue"
    ))


def handle_command_execution(response: Dict[str, Any], original_prompt: str) -> None:
    """Handle command execution with user confirmation"""
    commands = response.get("commands", [])
    description = response.get("description", "")
    warnings = response.get("warnings", [])
    
    if not commands:
        # If no commands, just show description
        console.print(Panel.fit(
            f"[bold blue]{get_message('ai_reply')}:[/bold blue]\n\n{description}",
            title=f"{get_message('reply')}: {original_prompt[:50]}{'...' if len(original_prompt) > 50 else ''}",
            border_style="blue"
        ))
        return
    
    # Show commands and description
    command_text = ""
    for i, cmd in enumerate(commands, 1):
        command_text += f"{i}. [bold cyan]{cmd}[/bold cyan]\n"
    
    info_text = f"[bold green]{get_message('recommended_commands')}:[/bold green]\n\n{command_text}\n"
    if description:
        info_text += f"[bold]{get_message('explanation')}:[/bold] {description}\n\n"
    
    if warnings:
        info_text += f"[bold yellow]{get_message('warnings')}:[/bold yellow]\n"
        for warning in warnings:
            info_text += f"  * {warning}\n"
        info_text += "\n"
    
    console.print(Panel.fit(
        info_text + f"[dim]{get_message('execute_commands')}[/dim]",
        title=f"{get_message('command_suggestions')}: {original_prompt[:50]}{'...' if len(original_prompt) > 50 else ''}",
        border_style="green"
    ))
    
    # Get user confirmation
    try:
        choice = input().strip().lower()
        if choice in ['y', 'yes', '是', 'y是']:
            execute_commands(commands)
        elif choice in ['s', 'selective', '选择']:
            execute_commands_selective(commands)
        else:
            console.print(f"[dim]{get_message('commands_cancelled')}[/dim]")
    except KeyboardInterrupt:
        console.print(f"\n[dim]{get_message('commands_cancelled')}[/dim]")


def execute_commands(commands: List[str]):
    """Execute all commands"""
    for i, cmd in enumerate(commands, 1):
        console.print(f"\n[bold]{get_message('executing_command')} {i}/{len(commands)}:[/bold] [cyan]{cmd}[/cyan]")
        try:
            result = subprocess.run(cmd, shell=True, capture_output=True, text=True)
            if result.returncode == 0:
                if result.stdout:
                    console.print(result.stdout)
                console.print(f"[green]{get_message('command_success')}[/green]")
            else:
                console.print(f"[red]{get_message('command_failed')} (返回码: {result.returncode})[/red]")
                if result.stderr:
                    console.print(f"[red]{get_message('error_info')}: {result.stderr}[/red]")
        except Exception as e:
            console.print(f"[red]{get_message('execution_error')}: {e}[/red]")


def execute_commands_selective(commands: List[str]):
    """Execute commands selectively"""
    for i, cmd in enumerate(commands, 1):
        console.print(f"\n[bold]{get_message('executing_command')} {i}/{len(commands)}:[/bold] [cyan]{cmd}[/cyan]")
        try:
            choice = input(f"{get_message('execute_command')}: ").strip().lower()
            if choice in ['q', 'quit', '退出']:
                console.print(f"[dim]{get_message('stopped_execution')}[/dim]")
                break
            elif choice in ['y', 'yes', '是']:
                result = subprocess.run(cmd, shell=True, capture_output=True, text=True)
                if result.returncode == 0:
                    if result.stdout:
                        console.print(result.stdout)
                    console.print(f"[green]{get_message('command_success')}[/green]")
                else:
                    console.print(f"[red]{get_message('command_failed')} (返回码: {result.returncode})[/red]")
                    if result.stderr:
                        console.print(f"[red]{get_message('error_info')}: {result.stderr}[/red]")
            else:
                console.print(f"[dim]{get_message('skipped_command')}[/dim]")
        except KeyboardInterrupt:
            console.print(f"\n[dim]{get_message('stopped_execution')}[/dim]")
            break
        except Exception as e:
            console.print(f"[red]{get_message('execution_error')}: {e}[/red]")



"""
AI client for communicating with AI API
"""

import requests
import json
from typing import Dict, Any, List
from .config import Config


class AIClient:
    """Client for communicating with AI API"""
    
    def __init__(self, config: Config):
        self.config = config
        self.default_model = config.get_model()  # Get model from config
        
    def send_request(self, prompt: str) -> Dict[str, Any]:
        """Send a request to the AI API"""
        api_endpoint = self.config.get_api_endpoint()
        api_key = self.config.get_api_key()
        
        if not api_endpoint or not api_key:
            raise RuntimeError("API endpoint and key must be configured first")
        
        # 验证端点格式
        if not self.config.validate_endpoint(api_endpoint):
            raise RuntimeError(f"Invalid API endpoint format: {api_endpoint}")
        
        # Prepare the request
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {api_key}"
        }
        
        # Get language setting for system prompt
        language = self.config.get_language()
        
        # System prompts in different languages
        system_prompts = {
            "zh": """你是一个有用的助手，专门提供终端命令建议。

你的回复应该是以下格式的 JSON：
{
    "commands": ["命令1", "命令2", ...],
    "description": "对这些命令的简要描述",
    "requires_input": false,
    "input_prompts": [],
    "warnings": ["警告1", "警告2", ...] (可选)
}

如果需要用户提供更多信息，请设置 "requires_input" 为 true 并在 "input_prompts" 中提供需要询问的问题。

专注于实用、安全的命令。始终提供实际可执行的命令，而不是解释说明。""",
            "en": """You are a helpful assistant that provides terminal command suggestions. 
                    
Your response should be structured as JSON with the following format:
{
    "commands": ["command1", "command2", ...],
    "description": "Brief description of what these commands do",
    "requires_input": false,
    "input_prompts": [],
    "warnings": ["warning1", "warning2", ...] (optional)
}

If you need more information from the user, set "requires_input" to true and provide "input_prompts" with questions to ask.

Focus on practical, safe commands. Always provide actual executable commands, not explanations."""
        }
        
        # Generic payload structure that should work with most AI APIs
        payload: Dict[str, Any] = {
            "model": self.default_model,
            "messages": [
                {
                    "role": "system",
                    "content": system_prompts.get(language, system_prompts["zh"])
                },
                {
                    "role": "user", 
                    "content": prompt
                }
            ],
            "max_tokens": 500,
            "temperature": 0.7
        }
        
        try:
            response = requests.post(
                api_endpoint,
                headers=headers,
                json=payload,
                timeout=30
            )
            
            if response.status_code == 200:
                result = response.json()
                # Handle OpenAI-style response
                if "choices" in result and len(result["choices"]) > 0:
                    content = result["choices"][0]["message"]["content"].strip()
                    return self._parse_ai_response(content)
                # Handle DeepSeek-style response
                elif "choices" in result and len(result["choices"]) > 0 and "text" in result["choices"][0]:
                    content = result["choices"][0]["text"].strip()
                    return self._parse_ai_response(content)
                # Handle other response formats
                elif "response" in result:
                    content = result["response"].strip()
                    return self._parse_ai_response(content)
                elif "content" in result:
                    content = result["content"].strip()
                    return self._parse_ai_response(content)
                else:
                    return {"error": f"Sorry, I couldn't understand the response format. Response: {result}"}
            else:
                # 详细的错误处理
                error_msg = self._format_api_error(response)
                raise RuntimeError(error_msg)
                
        except requests.exceptions.RequestException as e:
            raise RuntimeError(f"Failed to connect to AI API: {e}")
        except json.JSONDecodeError:
            raise RuntimeError("Invalid JSON response from AI API")
    
    def _format_api_error(self, response: requests.Response) -> str:
        """Format API error message with helpful suggestions"""
        status_code = response.status_code
        
        # 常见错误码的建议
        error_suggestions = {
            400: "请检查请求格式和参数",
            401: "请检查 API 密钥是否正确",
            403: "API 密钥没有权限，请检查账户状态",
            404: "API 端点不存在，请检查端点配置",
            429: "请求过于频繁，请稍后再试",
            500: "服务器内部错误，请稍后再试"
        }
        
        suggestion = error_suggestions.get(status_code, "请检查网络连接和配置")
        
        try:
            error_data = response.json()
            if "error" in error_data:
                detail = error_data["error"]
                if isinstance(detail, dict) and "message" in detail:
                    detail = detail["message"] # type: ignore
                return f"API 错误 {status_code}: {detail}\n建议: {suggestion}"
            elif "error_msg" in error_data:
                return f"API 错误 {status_code}: {error_data['error_msg']}\n建议: {suggestion}"
            elif "message" in error_data:
                return f"API 错误 {status_code}: {error_data['message']}\n建议: {suggestion}"
            else:
                return f"API 错误 {status_code}: {response.text}\n建议: {suggestion}"
        except json.JSONDecodeError:
            return f"API 错误 {status_code}: {response.text}\n建议: {suggestion}"
    
    def _parse_ai_response(self, content: str) -> Dict[str, Any]:
        """Parse AI response and extract commands"""
        try:
            # Try to parse as JSON first
            response_data = json.loads(content)
            if isinstance(response_data, dict):
                return response_data # type: ignore
        except json.JSONDecodeError:
            pass
        
        # If not JSON, try to extract commands from text
        lines = content.strip().split('\n')
        commands: List[str] = []
        description = ""
        
        for line in lines:
            line = line.strip()
            if line.startswith('`') and line.endswith('`'):
                # Extract command from backticks
                command = line[1:-1]
                if command:
                    commands.append(command)
            elif line.startswith('$ '):
                # Extract command after $ prompt
                command = line[2:]
                if command:
                    commands.append(command)
        
        # If no commands found, treat as description
        if not commands:
            description = content
        
        return {
            "commands": commands,
            "description": description or "Commands to execute",
            "requires_input": False,
            "input_prompts": [],
            "warnings": []
        }
